/*
 * packetmodeller/init.c
 *
 * Copyright (C) 2008-2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "init.h"
#include "netdevice.h"
#include "packet_modeller.h"
#include "pm_print.h"


/** Initialize vif configurations */
int
attach_vif_info()
{
    int count, i, j = 0;
    void * shm_pointer;
    unsigned char * mac;
    vifd_vif_info * vif_data;

    count = 0;
    memset(vif_list, 0, sizeof(vif_list));

    // initialize shared memory with vifd
    if((shmem_id = shmget(IPC_KEY, MAX_NUM_VIF * sizeof(vifd_vif_info), \
        0600)) < 0)
    {
        perror("shmget");
        pm_error("Can't attach to virtual interfaces. Make sure vifd is started.\n");
        return -1;
    }

    // attach vif data & get vif properties
    shm_pointer = shmat(shmem_id, NULL, SHM_RDONLY);
    for (i = 0; i < MAX_NUM_VIF; i++)
    {
        vif_data = (vifd_vif_info *) \
                    (shm_pointer + i * sizeof(struct vifd_vif_info));
        // check for valid config
        if (strncmp("IEEE 802.11", vif_data->vif_config.b.name, 11) == 0)
        {
            count++;
            vif_list[i].vif_data = vif_data;
            mac = vif_list[i].vif_mac;
            get_mac_by_dev(mac, (char *) &(vif_data->vif_name));
            sprintf(vif_list[j].vif_id, "%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x", \
                mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
            vif_list[i].vif_id[sizeof(host_id) - 1] = '\0';
        } else {
            vif_list[i].vif_data = NULL;
            vif_list[i].vif_id[0] = '\0';
        }
    }

    if (count == 0)
    {
        pm_error("No virtual interfaces found!\n");
    } else {
        char if_list[count * (IFNAMSIZ + 3)];
        char *ipf = if_list;
        for (i = 0; i < count; i++)
        {
            sprintf(ipf, " '%s'", vif_list[i].vif_data->vif_name);
            ipf += (3 + strlen(vif_list[i].vif_data->vif_name));
        }
        sprintf(ipf, "\n");
        pm_log("Using virtual interface(s):%s", if_list);
    }
    return count;
}

/** Initialize network file descriptors */
void
init_netdev()
{
    // initialize tap device (TODO: extend for multiple interfaces)
    if ((tap_fd = alloc_tap_socket(vif_list[0].vif_data->vif_name)) < 4)
    {
        pm_error("Open socket on '%s' failed\n", vif_list[0].vif_data->vif_name);
        exit(1);
    }
    pm_debug("init_netdev(): '%s' (tap_fd=%i)\n", \
        vif_list[0].vif_data->vif_name, tap_fd);

    // initialize network device
    if ((net_socket = alloc_net_socket(node_pif, ntohs(node_port))) < 4)
    {
        pm_error("Open socket on '%s' failed\n", node_pif);
        exit(1);
    }
    pm_debug("init_netdev(): '%s' (net_socket=%i)\n", node_pif, net_socket);
}

/** Initialize node variables */
int
pm_init()
{
    int i;

    // get host name
    if (gethostname(host_name, HOST_NAME_MAX) != 0)
    {
        pm_error("Unable to retrieve host name (%s)\n", strerror(errno));
        return -1;
    }

    // get physical MAC address
    get_mac_by_dev(node_pmac, node_pif);

    // create host ID from physical MAC address
    for (i = 0; i < sizeof(node_pmac); ++i)
    {
        sprintf((char *) node_id + 2 * i, "%02x", \
            ((unsigned char *) node_pmac)[i]);
    }
    node_id[sizeof(node_id) - 1] = '\0';

    // get host IP
    if (get_ip_by_dev(&node_ip, node_pif) != 0)
    {
        return -1;
    }

    // attach virtual interface configuration from vifd
    if ((num_vif = attach_vif_info()) < 1)
        return -1;

    // initialize network sockets
    init_netdev();



    printf("*************************************************************\n");
    printf("Node Name: %s\n", host_name);
    printf("Node ID: %s\n", node_id);
    printf("Node IP: %s\n", inet_ntoa(node_ip));
    printf("Node Port: %i\n", ntohs(node_port));
    printf("*************************************************************\n");

    return 0;
}
