/*
 * packetmodeller/message.c
 *
 * Copyright (C) 2008-2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "message.h"
#include "pm_print.h"


int
isWlanMessage(struct msg_header *header)
{
    return header->type == WLAN_MSG;
}

int
isConfigMessage(struct msg_header *header)
{
    return header->type == CONFIG_MSG;
}

int
isRegisterMessage(struct msg_header *header)
{
    return header->type == REGISTER_MSG;
}

int
isAcknowledgeMessage(struct msg_header *header)
{
    return header->type == ACK_MSG;
}

/** Print content of wlan_msg to debug output */
void
print_wlan_msg(const char *message)
{
    struct msg_header *header = (struct msg_header *) message;
    struct wlan_msg *msg = (struct wlan_msg *) message;
    int frame_size = msg->pkt_length;
    pm_debug("(%4i bytes) ----------- WLAN MESSAGE ----------------\n", \
        frame_size + WLAN_PREAMBLE_SIZE);
    pm_debug("(%4i bytes) msg_header ------------------------------\n", sizeof(struct msg_header));
    pm_debug("(%4i bytes) | type:\t\t%i\n", sizeof(header->type), header->type);
    pm_debug("(%4i bytes) | id:\t\t%s\n", sizeof(header->id), header->id);
    pm_debug("(%4i bytes) pkg_length:\t\t%i\n", sizeof(uint16_t), frame_size);
    struct ether_header *ether_hdr = (struct ether_header *) msg->wlan_pkt;
    pm_debug("(%4i bytes) raw_packet ------------------------------\n", frame_size);
    pm_debug("(%4i bytes) | ether_header --------------------------\n", sizeof(struct ether_header));
    pm_debug("(%4i bytes) | | ether_dhost\t%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx\n", \
        ETH_ALEN, ether_hdr->ether_dhost[0], ether_hdr->ether_dhost[1], \
        ether_hdr->ether_dhost[2], ether_hdr->ether_dhost[3], \
        ether_hdr->ether_dhost[4], ether_hdr->ether_dhost[5]);
    pm_debug("(%4i bytes) | | ether_shost\t%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx\n", \
        ETH_ALEN, ether_hdr->ether_shost[0], ether_hdr->ether_shost[1], \
        ether_hdr->ether_shost[2], ether_hdr->ether_shost[3], \
        ether_hdr->ether_shost[4], ether_hdr->ether_shost[5]);
    u_int16_t ether_type = ntohs(ether_hdr->ether_type);
    switch (ether_type)
    {
        case ETHERTYPE_IP:
            pm_debug("(%4i bytes) | | type\t\t%i (ETHERTYPE_IP)\n", sizeof(u_int16_t), ETHERTYPE_IP);
            break;
        case ETHERTYPE_ARP:
            pm_debug("(%4i bytes) | | type\t\t%i (ETHERTYPE_ARP)\n", sizeof(u_int16_t), ETHERTYPE_ARP);
            break;
        default:
            pm_debug("(%4i bytes) | | type\t\t%i (unknown)\n", sizeof(u_int16_t), ether_type);
            break;
    }
    pm_debug("------------------------------------------------------\n");
}

/** Print content of config_msg to debug output */
void
print_config_msg(const char *message)
{
    struct msg_header *header = (struct msg_header *) message;
    pm_debug("(%4i bytes) ----------- CONFIGURE MESSAGE -----------\n", CONFIG_MSG_SIZE);
    pm_debug("(%4i bytes) msg_header ------------------------------\n", sizeof(struct msg_header));
    pm_debug("(%4i bytes) | type:\t%i\n", sizeof(header->type), header->type);
    pm_debug("(%4i bytes) | id:\t%s\n", sizeof(header->id), header->id);
    struct config_msg *conf_msg = (struct config_msg *) message;
    switch (conf_msg->property)
    {
        case SIOCSIWFREQ:
            pm_debug("(%4i bytes) property:\t%4x (SIOCSIWFREQ)\n", sizeof(conf_msg->property), conf_msg->property);
            pm_debug("(%4i bytes) value:\t%.3e\n", sizeof(conf_msg->value), conf_msg->value.freq);
            break;
    }
    pm_debug("------------------------------------------------------\n");
}

/** Print content of ack_msg to debug output */
void
print_reg_msg(const char *message)
{
    struct msg_header *header = (struct msg_header *) message;
    pm_debug("(%4i bytes) ----------- REGISTER MESSAGE ------------\n", \
        REGISTER_MSG_SIZE);
    pm_debug("(%4i bytes) msg_header ------------------------------\n", sizeof(struct msg_header));
    pm_debug("(%4i bytes) | type:\t%i\n", sizeof(header->type), header->type);
    pm_debug("(%4i bytes) | id:\t%s\n", sizeof(header->id), header->id);
    struct register_msg *reg_msg = (struct register_msg *) message;
    pm_debug("(%4i bytes) msg_id:\t%i\n", sizeof(reg_msg->msg_id), reg_msg->msg_id);
    pm_debug("(%4i bytes) name:\t%s\n", sizeof(reg_msg->name), reg_msg->name);
    pm_debug("(%4i bytes) ip:\t\t%s\n", sizeof(reg_msg->ip), inet_ntoa(reg_msg->ip));
    pm_debug("(%4i bytes) port:\t%i\n", sizeof(reg_msg->port), ntohs(reg_msg->port));
    pm_debug("(%4i bytes) model_vif_info --------------------------\n", sizeof(model_vif_info));
    pm_debug("(%4i bytes) | vif_id:\t%s\n", sizeof(host_id), reg_msg->vif_info.vif_id);
    mac_addr *mac = (mac_addr *) reg_msg->vif_info.vif_mac;
    pm_debug("(%4i bytes) | vif_mac:\t%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x\n", \
        ETH_ALEN, ((unsigned char *) mac)[0], ((unsigned char *) mac)[1], \
        ((unsigned char *) mac)[2], ((unsigned char *) mac)[3], \
        ((unsigned char *) mac)[4], ((unsigned char *) mac)[5]);
    pm_debug("(%4i bytes) | vif_name:\t%s\n", IFNAMSIZ + 1, reg_msg->vif_info.vif_name);
    pm_debug("(%4i bytes) | freq:\t%.3e\n", sizeof(double), reg_msg->vif_info.freq);
    pm_debug("------------------------------------------------------\n");
}

/** Print content of ack_msg to debug output */
void
print_ack_msg(const char *message)
{
    struct msg_header *header = (struct msg_header *) message;
    pm_debug("(%4i bytes) ----------- ACKNOWLEDGE MESSAGE ---------\n", \
        ACK_MSG_SIZE);
    pm_debug("(%4i bytes) msg_header ------------------------------\n", sizeof(struct msg_header));
    pm_debug("(%4i bytes) | type:\t%i\n", sizeof(header->type), header->type);
    pm_debug("(%4i bytes) | id:\t%s\n", sizeof(header->id), header->id);
    struct ack_msg *msg = (struct ack_msg *) message;
    pm_debug("(%4i bytes) msg_id:\t%i\n", sizeof(uint8_t), msg->msg_id);
    pm_debug("------------------------------------------------------\n");
}

/** Print content of a message to debug output */
void
debug_msg(const char *message)
{
    struct msg_header *header = (struct msg_header *) message;
    switch (header->type)
    {
        case WLAN_MSG:
            print_wlan_msg(message);
            break;
        case CONFIG_MSG:
            print_config_msg(message);
            break;
        case REGISTER_MSG:
            print_reg_msg(message);
            break;
        case ACK_MSG:
            print_ack_msg(message);
            break;
        default:
            pm_debug("Warning: unknown message\n");
            return;
    }
}

/** LEGACY FUNCTIONS **********************************************************/

//~ /** Print raw data in hex to stdout */
//~ void
//~ dump_data(char *data, int len)
//~ {
    //~ int i;
//~
    //~ printf("(%d bytes) data: ", len);
    //~ for(i = 1; i <= len; i++)
    //~ {
        //~ printf("%02hhx ", *(data + (i-1)));
        //~ if (i % 16 == 0)
            //~ printf("\n                 ");
    //~ }
    //~ printf("\n");
//~ }


//~ /** Print IP header and return packet length */
//~ u_int16_t
//~ dump_ip_header(struct iphdr *ip_hdr)
//~ {
    //~ //structs and types defined in "netinet/ip.h"
    //~ printf("(%i bytes) header: ", sizeof(struct iphdr));
    //~ printf("tos 0x%x, ", ip_hdr->tos);
    //~ printf("ttl %i, ", ip_hdr->ttl);
    //~ printf("id %i, " , ntohs(ip_hdr->id));
    //~ printf("proto ");
    //~ switch (ip_hdr->protocol)
    //~ {
        //~ case 1:
            //~ printf("ICMP ");
            //~ break;
        //~ case 6:
            //~ printf("TCP ");
            //~ break;
        //~ case 17:
            //~ printf("UDP ");
            //~ break;
        //~ default:
            //~ //for more, see: /etc/protocols
            //~ printf("unknown ");
            //~ break;
    //~ }
    //~ printf("(%i), ", ip_hdr->protocol);
    //~ u_int16_t packet_length = ntohs(ip_hdr->tot_len);
    //~ printf("length %i\n", packet_length);
    //~ printf("                   ");
    //~ u_char ip[INET_ADDRSTRLEN];
    //~ inet_ntop(AF_INET, (struct in_addr *) &(ip_hdr->saddr), \
        //~ ip, INET_ADDRSTRLEN);
    //~ printf("saddr %s, ", ip);
    //~ inet_ntop(AF_INET, (struct in_addr *) &(ip_hdr->daddr), \
        //~ ip, INET_ADDRSTRLEN);
    //~ printf("daddr %s\n", ip);
//~
    //~ return packet_length;
//~ }
//~
//~ /** Print TCP packet */
//~ void
//~ dump_tcp_packet(char *tcp_packet, u_int16_t pkt_size)
//~ {
    //~ // structs and types defined in "netinet/tcp.h"
    //~ struct tcphdr *tcp_hdr = (struct tcphdr *) tcp_packet;
//~
    //~ printf("(%i bytes) tcp packet:\n", pkt_size);
    //~ printf("(%i bytes) header: ", sizeof(struct tcphdr));
    //~ printf("source %i, ", ntohs(tcp_hdr->source));
    //~ printf("dest %i\n", ntohs(tcp_hdr->dest));
    //~ dump_data(tcp_packet + sizeof(struct tcphdr), \
        //~ pkt_size - sizeof(struct tcphdr));
//~ }
//~
//~ /** Print IP packet */
//~ void
//~ dump_ip_packet(char* ip_packet, int pkt_size)
//~ {
    //~ struct iphdr *ip_hdr = (struct iphdr *) ip_packet;
//~
    //~ printf("(%i bytes) ip packet:\n", pkt_size);
    //~ if (pkt_size != dump_ip_header(ip_hdr))
    //~ {
        //~ printf("WARNING: ");
        //~ printf("defined packet size doesn't match with received packet size!\n");
    //~ }
    //~ u_int8_t proto = ip_hdr->protocol;
    //~ switch (proto)
    //~ {
        //~ case 6:
            //~ dump_tcp_packet(ip_packet + sizeof(struct iphdr), \
                //~ pkt_size - sizeof(struct iphdr));
            //~ break;
        //~ default:
            //~ dump_data(ip_packet + sizeof(struct iphdr), \
                //~ pkt_size - sizeof(struct iphdr));
            //~ break;
    //~ }
//~ }
//~
//~ /** Print PacketModeller <-> WlanModel message */
//~ void
//~ dump_msg(const char *pkt, int frame_length)
//~ {
    //~ char *wlan_packet;
//~
    //~ printf("****************************************\n");
    //~ printf("dump_msg():\n");
    //~ dump_header((struct msg_header *) pkt);
    //~ if (isNodeMessage((struct msg_header *) pkt))
        //~ dump_wlan_info((struct node_msg *) pkt);
    //~ if (isRegisterMessage((struct msg_header *) pkt))
        //~ dump_register_info((struct register_msg *) pkt);
    //~ if (isNodeMessage((struct msg_header *) pkt))
    //~ {
        //~ wlan_packet = (char *) &(*(struct node_msg *) pkt).wlan_pkt;
//~ //      dump_data(wlan_packet, frame_length);
    //~ }
    //~ if (isModelMessage((struct msg_header *) pkt))
    //~ {
        //~ wlan_packet = (char *) &(*(struct model_msg *) pkt).wlan_pkt;
//~ //      dump_data(wlan_packet, frame_length);
    //~ }
    //~ if (isModelMessage((struct msg_header *) pkt) || \
        //~ isNodeMessage((struct msg_header *) pkt))
    //~ {
        //~ printf("(%i bytes) ethernet packet:\n", frame_length);
        //~ u_int16_t ether_header_size = sizeof(struct ether_header);
        //~ u_int16_t payload_length = frame_length - ether_header_size;
        //~ u_int16_t type = dump_ethernet_header(wlan_packet);
        //~ switch (type)
        //~ {
            //~ case ETHERTYPE_IP:
                //~ dump_ip_packet(wlan_packet + ether_header_size, payload_length);
                //~ break;
            //~ default:
                //~ printf("****************************************\n");
                //~ dump_data(wlan_packet + ether_header_size, payload_length);
                //~ break;
        //~ }
    //~ }
    //~ printf("****************************************\n");
//~ }
