/*
 * vifd/vifd_ipc.c
 *
 * Copyright (C) 2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "vifd_ipc.h"
#include "vifd_print.h"

int
msgq_initialize(int key)
{
    int id;
    int msgq_flags = IPC_CREAT | 0664; // create read/write queue

    if ((id = msgget(key, msgq_flags)) < 0)
    {
        perror("msgget");
        exit(1);
    } else {
        return id;
    }
}

int
msgq_receive(int id, void * buffer, int size, int type)
{
    int rtrn;

    if ((rtrn = msgrcv(id, buffer, size, type, 0)) < 0)
    {
        perror("msgrcv"); // raise error, but continue execution
    }
    return rtrn;
}

int
msgq_delete(int id)
{
    return msgctl(id, IPC_RMID, NULL);
}

int
shmem_initialize(int key, int bytes)
{
    int id;
    int shmem_flags = IPC_CREAT | IPC_EXCL | 0600;

    if ((id = shmget(key, bytes, shmem_flags)) == -1)
    {
        if (errno != EEXIST)
        {
            perror("shmget");
        } else {  // if already exists, reattach and clean it
            if ((id = shmget(key, 0, 0600)) > -1)
            {
                memset(shmat(id, NULL, 0), 0, bytes);
            }
        }
    }

    return id;
}

char *
shmem_attach(int id, char * addr)
{
    char * return_addr;
    if ((return_addr = shmat(id, addr, 0)) == (char *) -1)
    {
        perror("shmat");
    }
    // evtl. add locking?

    return return_addr;
}

int
shmem_detach(char * addr)
{
    if (shmdt(addr) < 0)
    {
        perror("shmdt");
        return -1;
    }

    return 0;
}

int
shmem_delete(int id)
{
    return shmctl(id, IPC_RMID, NULL);
}
